@@ -6,31 +6,44 @@ import websocket
 from requests import ConnectionError, ReadTimeout
 from urllib3.connection import ConnectTimeoutError
 
-from devolo_home_control_api.backend.mprm_rest import MprmRest, MprmDeviceCommunicationError
+from .mprm_rest import MprmDeviceCommunicationError, MprmRest
 
 
 class MprmWebsocket(MprmRest):
     """
-    The MprmWebsocket object handles calls to the mPRM via websockets. It does not cover all API calls, just those
-    requested up to now. All calls are done in a gateway context, so you need to provide the ID of that gateway. As
-    it inherites from MprmRest, it is a singleton as well.
+    The abstract MprmWebsocket object handles calls to the mPRM via websockets. It does not cover all API calls, just those
+    requested up to now. All calls are done in a gateway context, so you have to create a derived class, that provides a
+    Gateway object and a Session object. Further, the derived class needs to implement methods to connect to the websocket,
+    either local or remote. Last but not least, the derived class needs to implement a method that is called on new messages.
 
-    :param gateway_id: Gateway ID (aka serial number), typically found on the label of the device
-    :param url: URL of the mPRM
+    The websocket connection itself runs in a thread, that might not terminate as expected. Using a with-statement is
+    recommended.
     """
 
-    def __init__(self, gateway_id: str, url: str):
-        super().__init__(gateway_id, url)
+    def __init__(self):
+        super().__init__()
         self._ws = None
         self._event_sequence = 0
 
-        self.publisher = None
-        self.on_update = None
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exception_type, exception_value, traceback):
+        self.websocket_disconnect()
 
 
-    def websocket_connection(self):
-        """ Set up the websocket connection """
-        ws_url = self._mprm_url.replace("https://", "wss://").replace("http://", "ws://")
+    def get_local_session(self):
+        raise NotImplementedError(f"{self.__class__.__name__} needs a method to connect locally to a gateway.")
+
+    def get_remote_session(self):
+        raise NotImplementedError(f"{self.__class__.__name__} needs a method to connect remotely to a gateway.")
+
+    def on_update(self, message):
+        raise NotImplementedError(f"{self.__class__.__name__} needs a method to process messages from the websocket.")
+
+    def websocket_connect(self):
+        """ Set up the websocket connection. """
+        ws_url = self._session.url.replace("https://", "wss://").replace("http://", "ws://")
         cookie = "; ".join([str(name) + "=" + str(value) for name, value in self._session.cookies.items()])
         ws_url = f"{ws_url}/remote/events/?topics=com/prosyst/mbs/services/fim/FunctionalItemEvent/PROPERTY_CHANGED," \
                  f"com/prosyst/mbs/services/fim/FunctionalItemEvent/UNREGISTERED" \
@@ -44,6 +57,11 @@ class MprmWebsocket(MprmRest):
                                           on_close=self._on_close)
         self._ws.run_forever(ping_interval=30, ping_timeout=5)
 
+    def websocket_disconnect(self):
+        """ Close the websocket connection. """
+        self._logger.info("Closing web socket connection.")
+        self._ws.close()
+
 
     def _on_close(self):
         """ Callback function to react on closing the websocket. """
@@ -61,7 +79,7 @@ class MprmWebsocket(MprmRest):
             self._try_reconnect(sleep_interval)
             sleep_interval = sleep_interval * 2 if sleep_interval < 2048 else 3600
 
-        self.websocket_connection()
+        self.websocket_connect()
 
     def _on_message(self, message: str):
         """ Callback function to react on a message. """
@@ -74,22 +92,20 @@ class MprmWebsocket(MprmRest):
             self._logger.warning("We missed a websocket message.")
             self._event_sequence = event_sequence
 
-        try:
-            self.on_update(message)
-        except TypeError:
-            self._logger.error("on_update is not set.")
+        self.on_update(message)
 
     def _on_open(self):
         """ Callback function to keep the websocket open. """
         def run(*args):
-            self._logger.info("Starting web socket connection")
+            self._logger.info("Starting web socket connection.")
             while self._ws.sock is not None and self._ws.sock.connected:
                 time.sleep(1)
         threading.Thread(target=run).start()
 
     def _try_reconnect(self, sleep_interval: int):
+        """ Try to reconnect to the websocket. """
         try:
-            self._logger.info("Trying to reconnect to the gateway.")
+            self._logger.info("Trying to reconnect to the websocket.")
             # TODO: Check if local_ip is still correct after lost connection
             self.get_local_session() if self._local_ip else self.get_remote_session()
             self.connected = True
